import numpy as np
import pylab
from lmfit import Minimizer, Parameters, fit_report

#nacteni dat do matice
data=np.loadtxt('Data.dat')  
xdata=data[:,0]
ydata=data[:,1]

def gaussian(x,p,w,g):
    return p/(g*2*np.pi) * np.exp(-(x-w)**2/(2*g**2))

#definice residualu: rozdilu fitovane funkce a dat
def residual(par, x, ydata):
    teor=gaussian(x, par['p'], par['w'],par['g']) +\
		gaussian(x, par['p2'], par['w2'],par['g2'])
    return teor-ydata    

#vytvoreni parametru a jejich startovacich hodnot. Mozno nastavit, zda se parametr fituje (vary=True) nebo ne 
params = Parameters()
params.add('p', value=7589,vary=True)
params.add('w', value=2,vary=True)
params.add('g', value=0.06597753,vary=True)

params.add('p2', value=1104,vary=True)
params.add('w2', value=1.8,vary=True)
params.add('g2', value=0.067,vary=True)

OriginalModel= ydata+residual(params,xdata,ydata)  # calculate the original model spectrum

#fitovani
minner = Minimizer(residual, params, fcn_args=(xdata,ydata))
result = minner.minimize(method='leastsq')
FinalModel = ydata + result.residual   # calculate final fit spectrum: data + residual
print(fit_report(result))  # write error report, alternative result.params.pretty_print()
#ulozeni statistiky do souboru
FileFit = open('Statistika.dat', 'w')
print (fit_report(result), file=FileFit)

#ulozeni nafitovaneho spektra do souboru
FileData= open('Spektrum.dat', 'w')
for i in range(len(xdata)):
    print (xdata[i], "\t",FinalModel[i], file=FileData)

# vykresleni fitu na monitor
pylab.plot(xdata, OriginalModel, 'g-', label='Original model')
pylab.plot(xdata, ydata, 'k-', label='data')
pylab.plot(xdata, FinalModel, 'r-', label='Fitted model')
legend = pylab.legend(loc='upper right', shadow=False, fontsize='x-small')
pylab.show()

