#program na flexibilni fitovani optickych spekter s dynamickym poctem ruznymi oscilatoru. momentalne ma implemenovane lorenzovku, komplexni lorentzovku, realnou gaussovku, lorentzovku a Voigtuv profil
#Ver 2023b, updated fitting to optical conductivity s12 
#Ver 2023, updated export to DF
#implementing erf*exp function for pump-probe spectroscopy

import numpy as np
#import pylab
import matplotlib.pyplot as plt
from lmfit import Minimizer, Parameters, fit_report
#from lmfit.models import  VoigtModel
from scipy.special import wofz
import time
from scipy import special


#definition of a gaussian function, pouze realne, pro fitovani ramana, luminiscence
def gaussian(x,p,w,g):
    return p/(g*2*np.pi) * np.exp(-(x-w)**2/(2*g**2))
    #return p/(g*2*math.pi) * (x-w)**2/(2*g)

#definition of a exponential decay
def ExpDec(x,p,t):
    return p*np.exp(x/t)

def ExpDecErf(x,p,t,tErf,dErf):
    return p*np.exp(x/t)*0.5*(special.erf((x-tErf)/dErf)+1)

#definition of normalized real Lorentzian, used for Raman or so
def LorentzianReal(x,p,w,g):
    return p*g/np.pi/((x-w)**2+g**2) 

#complex lorentzian, used for dielectric function
def lorentzian(x,p,w,g):
    return p**2/(w**2-x**2-1j*g*x) 

# lorentzian with complex spectral weight, assymetric oscillator
def LorentzianComplex(x,p,w,g,pc):
    return (p**2+1j*x*pc)/(w**2-x**2-1j*g*x) 

#  Voigt line shape at x with Lorentzian component HWHM gamma and Gaussian component HWHM alpha.
def Voigt(x, p,w,alpha, gamma):  
    sigma = alpha / np.sqrt(2 * np.log(2))
    return p*np.real(wofz((x-w + 1j*gamma)/sigma/np.sqrt(2))) / sigma/np.sqrt(2*np.pi)

# critical point model
def CritPoint(x, Ampl,Phase,Energy, Gamma,Power):  
	#Phase is in degrees
	return Ampl*np.exp(1j*np.radians(Phase))*(x-Energy+1j*Gamma)**Power;
	


#definition of model function used for fitting
def ResidualModelFunction(params, x, ydata, NumberOscilators):
	v = params.valuesdict()    #rozpakovani vsech hodnot parametru do v
	#postupny vypocet fitovaci funkce pomoci ruznych prispevku (oscilatoru). Zacina se s nulovou hodnotou
	e=0.0
	for n in range(NumberOscilators["eINF"]):  e = e+ v['eINF']   # pridani konstanty eINF
	for n in range(NumberOscilators["Lorentz"]): #pridani Lorenztovych oscilatoru
		e=e+lorentzian(x,v["p"+str(n+1)],v["w"+str(n+1)],v["g"+str(n+1)])        
	for n in range(NumberOscilators["Gauss"]): #pridani Gaussovych oscilatoru
		e=e+gaussian(x,v["Gp"+str(n+1)],v["Gw"+str(n+1)],v["Gg"+str(n+1)])   

	for n in range(NumberOscilators["ExpDec"]): #pridani exponencialniho poklesu
		e=e+ExpDec(x,v["Ep"+str(n+1)],v["Et"+str(n+1)])   
	for n in range(NumberOscilators["ExpDecErf"]): #Adding of exponential decay with erf onset
		e=e+ExpDecErf(x,v["Ep"+str(n+1)],v["Et"+str(n+1)],v["EtErf"+str(n+1)],v["EdErf"+str(n+1)])  
	for n in range(NumberOscilators["LorentzRe"]): #pridani realneho Lorentzova oscilatoru
		e=e+LorentzianReal(x,v["LRp"+str(n+1)],v["LRw"+str(n+1)],v["LRg"+str(n+1)])   
	for n in range(NumberOscilators["Voigt"]): #pridani realneho Voigtova oscilatoru
		e=e+Voigt(x,v["Vp"+str(n+1)],v["Vw"+str(n+1)],v["Valpha"+str(n+1)],v["Vgamma"+str(n+1)])   
	for n in range(NumberOscilators["LorentzC"]): #pridani Lorenztovky s komplexni vahou
		e=e+LorentzianComplex(x,v["LCp"+str(n+1)],v["LCw"+str(n+1)],v["LCg"+str(n+1)],v["LCwc"+str(n+1)])   
	for n in range(NumberOscilators["CritPoint"]): #adding critical point function
		e=e+CritPoint(x,v["Ampl"+str(n+1)],v["Phase"+str(n+1)],v["Energy"+str(n+1)],v["Gamma"+str(n+1)],v["Power"+str(n+1)])   
	if NumberOscilators["DataType"]=="s12": # selecting residual for fitting real and imag part of optical conductivity
		e	=-1j*x/60*(e-1)   #calculating optical conductivity from dielectric function, assumes x-axis is in wavenumbers
	return e   #returns the model function, which might be real or complex

def ResidualModel(params, x, ydata, NumberOscilators):
	e=ResidualModelFunction(params, x, ydata, NumberOscilators)
	if NumberOscilators["DataType"]=="R":  #volba residualni funkce, zde pro odrazivost R
		N = e**0.5      #vypocet indexu lomu z dielektricke funkce
		R=abs((N-1)/(N+1))**2   # Odrazivost 
		Resid=R-ydata     # vypocet residualu: rozdil funkce a dat v pripade odrazivosti
		#raw_input("Press the <ENTER> key to continue...")    
	if NumberOscilators["DataType"]=="Raman": #volba residualni funkce, zde pro Ramana nebo luminiscenci
		Raman= e.real  # fitovana funkce v pripade Ramanskych nebo luminiscencnich dat, 
		Resid=Raman-ydata     # vypocet residualu: rozdil funkce a dat v pripade Ramana potreba odremovat
		#raw_input("Press the <ENTER> key to continue...")    
	if NumberOscilators["DataType"]=="CP2Der": # selecting residual for fitting critical point to dielectric function (or its second derivative)
		Resid=np.abs(e-ydata)
	if NumberOscilators["DataType"]=="s12": # selecting residual for fitting real and imag part of optical conductivity
		#sigma	=-1j*x/60*(e-1)   #calculating optical conductivity from dielectric function, assumes x-axis is in wavenumbers
		Resid=np.abs(e - ydata)
	return Resid
    

def IsFitted(Str): # procedura na detekci zda se dany parametr fituje pomoci znaku "f"
	if (Str[0:1]=="F") or (Str[0:1]=="f"):
		Fit=False
		StrRed=float(Str[1:])
	else:
		Fit=True
		StrRed=float(Str)
	return 	Fit,StrRed  #vraci boolean Fit, ktery je false, jestli je to fixvany parametr,  a StrRed je hodnota parametru

#Generovani vstupnich parametru ze souboru, osetreni fixovani pomoci znaku "f" pred cislem
params = Parameters()  #zalozeni tridy parametru
#slovnik, ktery obsahuje vsechny typy a pocet danych oscilatoru. Zacina se s nulovyma hodnotama
NumberOscilators= {"DataType": "Raman","eINF": 0 , "Lorentz": 0,"Gauss": 0,"ExpDec": 0,"LorentzRe": 0,"Voigt": 0, "LorentzC":0,"ExpDecErf": 0, "CritPoint": 0} 
f = open('ParamsInput.dat', 'r')  #otevreni souboru se vstupnimi parametry
lines = f.readlines()    #nacteni souboru se vstupnimi parametry
lines = [x.strip('\t') for x in lines] # ocisteni od koncovych tabulatoru
for i in range(len(lines)):
	line=lines[i].split()   #rozdeleni radku na jednotive stringy
	if len(line)<2: break  #pokud je prazdna radka, tak konci cteni, za ni muzou byt nepouzite oscilatory
	if line[0]=='DataType':  #nacteni typu data, jako Raman, R (odrazivost) atp. 
		NumberOscilators["DataType"]= line[1]
	if line[0]=='eINF':  #zarazeni konstanty eINF
		NumberOscilators["eINF"]=NumberOscilators["eINF"]+1        # zvysuje pocet daneho oscilatoru v prehledu oscilatoru 
		Fitted,Val=IsFitted(line[1])        #nacteni hodnoty a osetreni znaku "f"
		params.add('eINF', value=Val,vary=Fitted)     #generace parametru, jeho hodnoty a toho, zda se fituje
	if line[0]=='Lorentz': #zarazeni Lorenztova oscilatoru
		NumberOscilators["Lorentz"]=NumberOscilators["Lorentz"]+1    # zvysuje pocet daneho oscilatoru v prehledu oscilatoru 
		OscNum=NumberOscilators["Lorentz"]        
		Fitted,Val=IsFitted(line[1])        
		params.add('p'+str(OscNum), value=Val,vary=Fitted)
		Fitted,Val=IsFitted(line[2])
		params.add('w'+str(OscNum), value=Val,vary=Fitted)
		Fitted,Val=IsFitted(line[3])
		params.add('g'+str(OscNum), value=Val,vary=Fitted)
	if line[0]=='Gauss': #zarazeni Gaussova oscilatoru
		NumberOscilators["Gauss"]=NumberOscilators["Gauss"]+1    # zvysuje pocet daneho oscilatoru v prehledu oscilatoru 
		OscNum=NumberOscilators["Gauss"]       
		Fitted,Val=IsFitted(line[1])        
		params.add('Gp'+str(OscNum), value=Val,vary=Fitted)
		Fitted,Val=IsFitted(line[2])
		params.add('Gw'+str(OscNum), value=Val,vary=Fitted)
		Fitted,Val=IsFitted(line[3])
		params.add('Gg'+str(OscNum), value=Val,vary=Fitted)

	if line[0]=='ExpDec': #zarazeni Exponencialniho poklesu
		NumberOscilators["ExpDec"]=NumberOscilators["ExpDec"]+1    # zvysuje pocet daneho oscilatoru v prehledu oscilatoru 
		OscNum=NumberOscilators["ExpDec"]       
		Fitted,Val=IsFitted(line[1])        
		params.add('Ep'+str(OscNum), value=Val,vary=Fitted)
		Fitted,Val=IsFitted(line[2])
		params.add('Et'+str(OscNum), value=Val,vary=Fitted)
		
	if line[0]=='ExpDecErf': #addin of exponential decay with Erf onset
		NumberOscilators["ExpDecErf"]=NumberOscilators["ExpDecErf"]+1    # zvysuje pocet daneho oscilatoru v prehledu oscilatoru 
		OscNum=NumberOscilators["ExpDecErf"]       
		Fitted,Val=IsFitted(line[1])        
		params.add('Ep'+str(OscNum), value=Val,vary=Fitted)
		Fitted,Val=IsFitted(line[2])
		params.add('Et'+str(OscNum), value=Val,vary=Fitted)
		Fitted,Val=IsFitted(line[3])
		params.add('EtErf'+str(OscNum), value=Val,vary=Fitted)
		Fitted,Val=IsFitted(line[4])
		params.add('EdErf'+str(OscNum), value=Val,vary=Fitted)

	if line[0]=='LorentzRe': #zarazeni realneho Lorentzova oscilatoru, pro Ramana
		NumberOscilators["LorentzRe"]=NumberOscilators["LorentzRe"]+1    # zvysuje pocet daneho oscilatoru v prehledu oscilatoru 
		OscNum=NumberOscilators["LorentzRe"]
		Fitted,Val=IsFitted(line[1])        
		params.add('LRp'+str(OscNum), value=Val,vary=Fitted)
		Fitted,Val=IsFitted(line[2])
		params.add('LRw'+str(OscNum), value=Val,vary=Fitted)
		Fitted,Val=IsFitted(line[3])
		params.add('LRg'+str(OscNum), value=Val,vary=Fitted)

	if line[0]=='Voigt': #zarazeni realneho Vightova profilu pro Ramana, ma ctyri parametru
		NumberOscilators["Voigt"]=NumberOscilators["Voigt"]+1    # zvysuje pocet daneho oscilatoru v prehledu oscilatoru 
		OscNum=NumberOscilators["Voigt"]
		Fitted,Val=IsFitted(line[1])        
		params.add('Vp'+str(OscNum), value=Val,vary=Fitted)
		Fitted,Val=IsFitted(line[2])
		params.add('Vw'+str(OscNum), value=Val,vary=Fitted)
		Fitted,Val=IsFitted(line[3])
		params.add('Valpha'+str(OscNum), value=Val,vary=Fitted)
		Fitted,Val=IsFitted(line[4])
		params.add('Vgamma'+str(OscNum), value=Val,vary=Fitted)
	if line[0]=='LorentzC': #zarazeni Lorentzova oscilatoru s komplexni vahou
		NumberOscilators["LorentzC"]=NumberOscilators["LorentzC"]+1    # zvysuje pocet daneho oscilatoru v prehledu oscilatoru 
		OscNum=NumberOscilators["LorentzC"]
		Fitted,Val=IsFitted(line[1])        
		params.add('LCp'+str(OscNum), value=Val,vary=Fitted)
		Fitted,Val=IsFitted(line[2])
		params.add('LCw'+str(OscNum), value=Val,vary=Fitted)
		Fitted,Val=IsFitted(line[3])
		params.add('LCg'+str(OscNum), value=Val,vary=Fitted)
		Fitted,Val=IsFitted(line[4])
		params.add('LCwc'+str(OscNum), value=Val,vary=Fitted)
	if line[0]=='CritPoint': # adding critical point model
		NumberOscilators["CritPoint"]=NumberOscilators["CritPoint"]+1    # zvysuje pocet daneho oscilatoru v prehledu oscilatoru 
		OscNum=NumberOscilators["CritPoint"]
		Fitted,Val=IsFitted(line[1])        
		params.add('Ampl'+str(OscNum), value=Val,vary=Fitted)
		Fitted,Val=IsFitted(line[2])
		params.add('Phase'+str(OscNum), value=Val,vary=Fitted)
		Fitted,Val=IsFitted(line[3])
		params.add('Energy'+str(OscNum), value=Val,vary=Fitted)
		Fitted,Val=IsFitted(line[4])
		params.add('Gamma'+str(OscNum), value=Val,vary=Fitted)
		Fitted,Val=IsFitted(line[5])
		params.add('Power'+str(OscNum), value=Val,vary=Fitted)


#data loading
if (NumberOscilators["DataType"]=="CP2Der") or (NumberOscilators["DataType"]=="s12"): # 
	data=np.loadtxt('data.dat',skiprows=3)   #skipping 3 lines of the header of Woollam mat files
	xdata=data[:,0]
	ydata=data[:,1] + 1j*data[:,2]    # making ydata a complex dielectric function
	
	
else:
	data=np.loadtxt('data.dat') 
	xdata=data[:,0]
	ydata=data[:,1]
	global xdataGl,ydataGl
	xdataGl=data[:,0]
	ydataGl=data[:,1]

       
#rutina ktera se vola po kazde iteraci a vykresluje staf fitovane fuknce,  Potrebuje pracovat s plt.ion(), 
def PlotCurrentInteration(params, it, resid,*args, **kws):
	lines=plt.plot(xdataGl, ydataGl+resid, 'b-',label='fitting')
	l = lines.pop(0)
	#plt.pause(0.01)  #pokud se toto pause zaremuje, tak to ploti priliz rychle
	print ("iteration", it)
	l.remove()
	return    

# plotting data
if (NumberOscilators["DataType"]=="CP2Der") or (NumberOscilators["DataType"]=="s12"): 
	fig, (ax1, ax2) = plt.subplots(1, 2)	#definition of the figure
	ax1.plot(xdata, np.real(ydata), 'k-',label='Re Data')
	ax2.plot(xdata, np.imag(ydata), 'k-',label='Im Data')
	original= ResidualModelFunction(params,xdata,ydata,NumberOscilators)
	ax1.plot(xdata, np.real(original), 'b.',label='Re Start Model')
	ax2.plot(xdata, np.imag(original), 'b.',label='Im Start Model')
else:
	plt.plot(xdataGl, ydataGl, 'k-',label='Data')
	original= ResidualModelFunction(params,xdata,ydata,NumberOscilators)
	#plt.plot(xdata, original, 'g-', label='Original model')
	#plt.legend(loc='upper right', shadow=False)

#plt.ion()
#plt.pause(0.05)
#plt.show()
#raw_input("Press <ENTER> to start fitting...")    

print ("Start Fitting:")
TimeStart=time.time()

#fitovani
#minner = Minimizer(ResidualModel, params, fcn_args=(xdata,ydata,NumberOscilators), iter_cb=PlotCurrentInteration)  # u tohoto fitovani se zpousti za kazdou iteraci procedura PlotCurrentInteration. Mozno vykreslovat data behem fitovani
minner = Minimizer(ResidualModel, params, fcn_args=(xdata,ydata,NumberOscilators))  #fitovani bez prubezneho zobrazovani

result = minner.minimize()  #zde se uchovovaji vsechny vysledky fitovani
final = ydata + result.residual # finalni spektrum: data + residual
OutParams=result.params

TimeEnd=time.time()    
print ("Cas minimalizace: ", TimeEnd-TimeStart , " s")
print

#vytisteni statistiky na obrazovku a do souboru
print(fit_report(result,min_correl=0.5))   
FileFit = open('Statistika.dat', 'w+')
print  (fit_report(result,min_correl=0.5), file=FileFit)   

#ulozeni nafitovaneho spektra do souboru
FileData = open('Spektrum.dat', 'w+')
for i in range(len(xdata)):  print (xdata[i],final[i], file=FileData)


DF=ResidualModelFunction(OutParams, xdata, ydata, NumberOscilators)

if NumberOscilators["DataType"]=="R": 
	FileDF = open('DF.dat', 'w+')
	for i in range(len(xdata)):  
			print (xdata[i], "\t", np.real(DF[i]), "\t", np.imag(DF[i]), file=FileDF)


OscArray=xdata	#pole kde se budou ukladat individualni spektra oscilatoru. Prvni sloupec jsou frekvence z dat
#ulozeni obdrzenych parametru do souboru v tomsamem formatu jako ma ten vstupni soubor, mozno jednoduse zkopirovat
FileParOut = open('ParamsOutput.dat', 'w')

def StrForm(strval):    #funkce na formatovani vystupniho cisla na 4 platne cislice a prevod do stringu
	return str("{0:.4g}".format(strval))
print ("DataType\t" + NumberOscilators["DataType"], file=FileParOut)
for i in range(NumberOscilators["eINF"]): 
        word="eINF"+"\t"
        if OutParams["eINF"].vary==False:word=word+"f"          #pokud je parametr fixovany, priadava "f"
        word=word+StrForm(OutParams["eINF"].value)
        print (word, file=FileParOut)
        OscArray=np.c_[OscArray,np.full(len(xdata), OutParams["eINF"].value)]
for i in range(NumberOscilators["Lorentz"]): 
        word="Lorentz"+"\t"
        if OutParams['p'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['p'+str(i+1)].value)+"\t"
        if OutParams['w'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['w'+str(i+1)].value)+"\t"
        if OutParams['g'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['g'+str(i+1)].value)
        print (word, file=FileParOut)
for i in range(NumberOscilators["Gauss"]): 
        word="Gauss"+"\t"
        if OutParams['Gp'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['Gp'+str(i+1)].value)+"\t"
        if OutParams['Gw'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['Gw'+str(i+1)].value)+"\t"
        if OutParams['Gg'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['Gg'+str(i+1)].value)
        print (word, file=FileParOut)
        OscArray=np.c_[OscArray,gaussian(xdata,OutParams['Gp'+str(i+1)].value,OutParams['Gw'+str(i+1)].value,OutParams['Gg'+str(i+1)].value)]
		
for i in range(NumberOscilators["ExpDec"]): 
        word="ExpDec"+"\t"
        if OutParams['Ep'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['Ep'+str(i+1)].value)+"\t"
        if OutParams['Et'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['Et'+str(i+1)].value)
        print (word, file=FileParOut)
		
for i in range(NumberOscilators["ExpDecErf"]): 
        word="ExpDecErf"+"\t"
        if OutParams['Ep'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['Ep'+str(i+1)].value)+"\t"
        if OutParams['Et'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['Et'+str(i+1)].value)+"\t"
        if OutParams['EtErf'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['EtErf'+str(i+1)].value)+"\t"
        if OutParams['EdErf'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['EdErf'+str(i+1)].value)
        print (word, file=FileParOut)


for i in range(NumberOscilators["LorentzRe"]): 
        word="LorentzRe"+"\t"
        if OutParams['LRp'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['LRp'+str(i+1)].value)+"\t"
        if OutParams['LRw'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['LRw'+str(i+1)].value)+"\t"
        if OutParams['LRg'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['LRg'+str(i+1)].value)
        print (word, file=FileParOut)
for i in range(NumberOscilators["Voigt"]): 
        word="Voigt"+"\t"
        if OutParams['Vp'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['Vp'+str(i+1)].value)+"\t"
        if OutParams['Vw'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['Vw'+str(i+1)].value)+"\t"
        if OutParams['Valpha'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['Valpha'+str(i+1)].value)+"\t"
        if OutParams['Vgamma'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['Vgamma'+str(i+1)].value)
        print (word, file=FileParOut)
for i in range(NumberOscilators["LorentzC"]): 
        word="LorentzC"+"\t"
        if OutParams['LCp'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['LCp'+str(i+1)].value)+"\t"
        if OutParams['LCw'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['LCw'+str(i+1)].value)+"\t"
        if OutParams['LCg'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['LCg'+str(i+1)].value)+"\t"
        if OutParams['LCwc'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['LCwc'+str(i+1)].value)
        print (word, file=FileParOut)

for i in range(NumberOscilators["CritPoint"]): 
        word="CritPoint"+"\t"
        if OutParams['Ampl'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['Ampl'+str(i+1)].value)+"\t"
        if OutParams['Phase'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['Phase'+str(i+1)].value)+"\t"
        if OutParams['Energy'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['Energy'+str(i+1)].value)+"\t"
        if OutParams['Gamma'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['Gamma'+str(i+1)].value)+"\t"
        if OutParams['Power'+str(i+1)].vary==False:word=word+"f"
        word=word+StrForm(OutParams['Power'+str(i+1)].value)
        print (word, file=FileParOut)


#ulozeni individualnich oscilatoru do souboru
with open("Oscillators.dat", 'w') as file_handler: np.savetxt(file_handler,OscArray, fmt='%.4g',delimiter='\t')
		
FileFit.close()		
FileData.close()		
FileParOut.close()		

if (NumberOscilators["DataType"]=="CP2Der") or (NumberOscilators["DataType"]=="s12")  : 
	final= ResidualModelFunction(OutParams,xdata,ydata,NumberOscilators)
	ax1.set_title('Re')
	ax2.set_title('Im')
	ax1.plot(xdata, np.real(final), 'r-',label='Re Final Model')
	ax2.plot(xdata, np.imag(final), 'r-',label='Im Final Model')
	plt.legend(loc='upper right', shadow=False)
	plt.show()


else:
	plt.plot(xdata, final, 'r-',label='Fitted model')	#vykresli nafitovane spektrum
	#plt.plot(xdata,OscArray[:,2:], 'b-',label='oscillators')   #vykresli prispevky individualnich oscilatoru
	plt.legend(loc='upper right', shadow=False)
	plt.show()



